###################
# Locals
###################

locals {
  # We do the following to be sure that the VPC config has been updated to label the subnets with the cluster name.
  cluster_name = var.cluster_config.cluster_name
  tags         = var.cluster_config.tags
  cluster_tags = merge(local.tags, {
    managed-by  = "terraform"
    ClusterName = local.cluster_name
  })
  vpc_security_group_ids = var.cluster_config.managed_node_group.vpc_security_group_ids
  managed_node_group_name = var.cluster_config.managed_node_group.name == null ? "${local.cluster_name}-standard" : var.cluster_config.managed_node_group.name
}

###################
# Create IAM Managed Node Group Role
###################

module "managed_node_group_role" {
  source = "../eks-node-role"

  cluster_name = local.cluster_name
}

###################
# Create EKS Cluster without A Managed Node Group
###################

module "eks" {
  source  = "terraform-aws-modules/eks/aws"
  version = "19.5.1"

  cluster_name                    = local.cluster_name
  cluster_version                 = var.cluster_config.cluster_version
  cluster_endpoint_private_access = var.cluster_config.cluster_endpoint_private_access
  cluster_endpoint_public_access  = var.cluster_config.cluster_endpoint_public_access
  cluster_enabled_log_types       = var.cluster_config.cluster_enabled_log_types
  cluster_timeouts                = var.cluster_config.cluster_timeouts

  cluster_security_group_additional_rules = {
    # Additional ports required because EKS module version 18.0+ changes how SGs are configured.
    # See https://github.com/terraform-aws-modules/terraform-aws-eks/issues/1748 for details
    egress_nodes_ephemeral_ports_tcp = {
      description                = "To node 1025-65535"
      protocol                   = "tcp"
      from_port                  = 1025
      to_port                    = 65535
      type                       = "egress"
      source_node_security_group = true
    }
  }

  vpc_id     = var.cluster_config.vpc_id
  subnet_ids = var.cluster_config.private_subnets
  tags       = local.cluster_tags
}

###################
# Create Managed Node Group
###################

module "eks_managed_node_group" {
  source  = "terraform-aws-modules/eks/aws//modules/eks-managed-node-group"
  version = "19.5.1"

  name            = local.managed_node_group_name
  cluster_name    = local.cluster_name
  cluster_version = var.cluster_config.cluster_version
  create_iam_role = false
  iam_role_arn    = module.managed_node_group_role.arn

  // The following variables are necessary if you decide to use the module outside of the parent EKS module context.
  // Without it, the security groups of the nodes are empty and thus won't join the cluster.
  cluster_primary_security_group_id = module.eks.node_security_group_id

  subnet_ids             = var.cluster_config.private_subnets
  min_size               = var.cluster_config.managed_node_group.min_size
  desired_size           = var.cluster_config.managed_node_group.desired_size
  max_size               = var.cluster_config.managed_node_group.max_size
  ami_type               = var.cluster_config.managed_node_group.ami_type
  instance_types         = var.cluster_config.managed_node_group.instance_types
  vpc_security_group_ids = local.vpc_security_group_ids

  capacity_type         = var.cluster_config.managed_node_group.capacity_type
  block_device_mappings = {
    xvda = {
      device_name = "/dev/xvda"
      ebs = {
        volume_size           = var.cluster_config.managed_node_group.volume_size
        volume_type           = var.cluster_config.managed_node_group.volume_type
        iops                  = var.cluster_config.managed_node_group.iops
        throughput            = var.cluster_config.managed_node_group.throughput
        encrypted             = true
        delete_on_termination = true
      }
    }
  }

  depends_on = [
    module.managed_node_group_role,
  ]
}
