## Overview

This module can be used to create an IAM role for Nodes used in an AWS EKS cluster. The module
is preconfigured with the default IAM polices required for a node to have
the right permissions to function.  You can also use the `additional_node_policies`
variable to pass additional polices to attach to the node role.

Usage:
```terraform
module "karpenter_node_role" {
  source = "..."

  cluster_name               = local.cluster_name
  node_role_name             = "${local.cluster_name}-karpenter-node"
  
  additional_node_policies = {
    "AmazonSSMManagedInstanceCore" : "arn:aws:iam::aws:policy/AmazonSSMManagedInstanceCoreV2"
    "NodeSecurityPolicy": "arn:aws:iam::123456789012:policy/my-node-security-policy"
  }
  tags = {
    env = Dev
  }
}
```

Assuming that `local.cluster_name == "alice"`, then the above module usage will create a IAM role with the
name `alice-karpenter-node` and it will have 5 policies attached.  Note, that because the `additional_node_policies`
variable lists a key, AmazonSSMManagedInstanceCore, that is in the `default_node_policies` lists, the value
`arn:aws:iam::aws:policy/AmazonSSMManagedInstanceCoreV2` will be used instead of the default policy,
`arn:aws:iam::aws:policy/AmazonSSMManagedInstanceCore`.

**Warning**

If you don't add the role to the `aws-auth` ConfigMap in the kube-system namespace like in the following usage then
you will have nodes that fail to successfully join the cluster.

```yaml
apiVersion: v1
kind: ConfigMap
metadata:
  name: aws-auth
  namespace: kube-system
data:
  mapRoles: |
    - groups:
      - system:bootstrappers
      - system:nodes
      rolearn: arn:aws:iam::763916856451:role/alice-karpenter-node
      username: system:node:{{EC2PrivateDNSName}}
    - ...
```

## Requirements

| Name | Version |
|------|---------|
| <a name="requirement_terraform"></a> [terraform](#requirement\_terraform) | >= 1.3.0 |
| <a name="requirement_aws"></a> [aws](#requirement\_aws) | >= 4.34.0 |

## Providers

| Name | Version |
|------|---------|
| <a name="provider_aws"></a> [aws](#provider\_aws) | >= 4.34.0 |

## Modules

No modules.

## Resources

| Name | Type |
|------|------|
| [aws_iam_role.nodes](https://registry.terraform.io/providers/hashicorp/aws/latest/docs/resources/iam_role) | resource |
| [aws_iam_role_policy_attachment.this](https://registry.terraform.io/providers/hashicorp/aws/latest/docs/resources/iam_role_policy_attachment) | resource |
| [aws_iam_policy_document.assume_role_policy](https://registry.terraform.io/providers/hashicorp/aws/latest/docs/data-sources/iam_policy_document) | data source |
| [aws_partition.current](https://registry.terraform.io/providers/hashicorp/aws/latest/docs/data-sources/partition) | data source |

## Inputs

| Name                                                                                                           | Description | Type | Default | Required |
|----------------------------------------------------------------------------------------------------------------|-------------|------|---------|:--------:|
| <a name="input_additional_node_policies"></a> [additional\_node\_policies](#input\_additional\_node\_policies) | A map of keys to ARNs of additional polices to attach to the node role.  Matching keys in this list will override the value in the default\_node\_policies variable. | `map(string)` | `{}` | no |
| <a name="input_cluster_name"></a> [cluster\_name](#input\_cluster\_name)                                       | The name of the EKS Cluster | `string` | n/a | yes |
| <a name="input_default_node_policies"></a> [default\_node\_policies](#input\_default\_node\_policies)          | A map of keys to ARNs of default polices to attach to the node role | `map(string)` | <pre>{<br>  "AmazonEC2ContainerRegistryReadOnly": "arn:aws:iam::aws:policy/AmazonEC2ContainerRegistryReadOnly",<br>  "AmazonEKSWorkerNodePolicy": "arn:aws:iam::aws:policy/AmazonEKSWorkerNodePolicy",<br>  "AmazonEKS_CNI_Policy": "arn:aws:iam::aws:policy/AmazonEKS_CNI_Policy",<br>  "AmazonSSMManagedInstanceCore": "arn:aws:iam::aws:policy/AmazonSSMManagedInstanceCore"<br>}</pre> | no |
| <a name="input_node_role_name"></a> [node\_role\_name](#input\_node\_role\_name)                               | The name of the EKS Cluster Node role | `string` | `null` | no |
| <a name="input_tags"></a> [tags](#input\_tags)                                                                 | A map of tags to add to resources | `map(string)` | `null` | no |

## Outputs

| Name | Description |
|------|-------------|
| <a name="output_arn"></a> [arn](#output\_arn) | The ARN of the Node role |
| <a name="output_name"></a> [name](#output\_name) | The name of the Node role |

